/*
 * main.h
 *
 * PWLib application header file for opengk
 *
 * Copyright 2001 Equivalence Pty. Ltd.
 *
 * $Log: main.h,v $
 * Revision 1.14  2006/04/25 09:10:04  rjongbloed
 * Ported gatekeeper server to (optionally) use OPAL code base
 *
 * Revision 1.13  2004/12/15 06:29:53  csoutheren
 * Added OSP support
 *
 * Revision 1.12  2004/12/14 11:15:56  rjongbloed
 * Added extra parameter to TranslateAliasAddress for OSP support
 *
 * Revision 1.11  2004/08/15 07:19:26  rjongbloed
 * Fixed compatibility with gkserver changes
 *
 * Revision 1.10  2003/04/02 06:13:02  robertj
 * Added H.323 Annex G Peer Element support to gatekeeper server.
 *
 * Revision 1.9  2003/04/01 07:30:05  robertj
 * Added function to translate alias into alias list and transport to make
 *   use of full information returned by peer element.
 *
 * Revision 1.8  2003/03/18 00:01:11  craigs
 * Added support for H.501 clearing houses
 *
 * Revision 1.7  2003/02/02 01:47:54  robertj
 * Added ability to set static route maps for aliases to gateway hosts.
 *
 * Revision 1.6  2002/10/21 03:12:05  robertj
 * Added unregister an endpoint or clear a call buttons to status page.
 *
 * Revision 1.5  2002/07/17 09:19:04  robertj
 * Allowed for not having gif file for product name in default header.
 * Changed to use default pwlib page graphic after above changes.
 *
 * Revision 1.4  2002/05/06 00:48:46  robertj
 * Sizeable rewrite of gatekeeper server code to make more bulletproof against
 *   multithreaded operation. Especially when using slow response/RIP feature.
 * Also changed the call indexing to use call id and direction as key.
 *
 * Revision 1.3  2002/02/04 05:19:40  robertj
 * Added call status display on HTTP.
 * Added test for slow ARQ
 *
 * Revision 1.2  2001/08/10 11:03:49  robertj
 * Major changes to H.235 support in RAS to support server.
 *
 * Revision 1.1  2001/08/03 04:48:57  robertj
 * First cut of a gatekeeper
 *
 */

#ifndef _OpenGk_MAIN_H
#define _OpenGk_MAIN_H


#include <ptlib.h>
#include <ptclib/httpsvc.h>

#ifdef USE_OPAL

#include <h323/h323ep.h>
#include <h323/gkserver.h>

#else

#include <h323.h>
#include <gkserver.h>

#endif


class MyEndPoint;
class MyGatekeeperServer;


class MyGatekeeperCall : public H323GatekeeperCall
{
  PCLASSINFO(MyGatekeeperCall, H323GatekeeperCall);
  public:
    MyGatekeeperCall(
      MyGatekeeperServer & server,
      const OpalGloballyUniqueID & callIdentifier, /// Unique call identifier
      Direction direction
    );
    ~MyGatekeeperCall();

    virtual H323GatekeeperRequest::Response OnAdmission(
      H323GatekeeperARQ & request
    );

#ifdef H323_TRANSNEXUS_OSP
    bool AuthoriseOSPCall(H323GatekeeperARQ & info);
    OpalOSP::Transaction * ospTransaction;
#endif
};



class MyGatekeeperServer : public H323GatekeeperServer
{
    PCLASSINFO(MyGatekeeperServer, H323GatekeeperServer);
  public:
    MyGatekeeperServer(MyEndPoint & ep);

    // Overrides
    virtual H323GatekeeperCall * CreateCall(
      const OpalGloballyUniqueID & callIdentifier,
      H323GatekeeperCall::Direction direction
    );
    virtual bool TranslateAliasAddress(
      const H225_AliasAddress & alias,
      H225_ArrayOf_AliasAddress & aliases,
      H323TransportAddress & address,
      bool & isGkRouted,
      H323GatekeeperCall * call
    );

    // new functions
    bool Initialise(PConfig & cfg, PConfigPage * rsrc);

#ifdef H323_TRANSNEXUS_OSP
    OpalOSP::Provider * GetOSPProvider() const
    { return ospProvider; }
#endif

  private:
    MyEndPoint & endpoint;

    class RouteMap : public PObject {
        PCLASSINFO(RouteMap, PObject);
      public:
        RouteMap(
          const PString & alias,
          const PString & host
        );
        RouteMap(
          const RouteMap & map
        ) : alias(map.alias), regex(map.alias), host(map.host) { }

        void PrintOn(
          ostream & strm
        ) const;

        bool IsValid() const;

        bool IsMatch(
          const PString & alias
        ) const;

        const H323TransportAddress & GetHost() const { return host; }

      private:
        PString              alias;
        PRegularExpression   regex;
        H323TransportAddress host;
    };
    PList<RouteMap> routes;

    PMutex reconfigurationMutex;

#ifdef H323_TRANSNEXUS_OSP
    OpalOSP::Provider * ospProvider;
    PString ospRoutingURL;
    PString ospPrivateKeyFileName;
    PString ospPublicKeyFileName;
    PString ospServerKeyFileName;
#endif
};


class MyEndPoint : public H323EndPoint
{
    PCLASSINFO(MyEndPoint, H323EndPoint);
  public:
    MyEndPoint(
#ifdef USE_OPAL
      OpalManager & mgr
#endif
    );
    ~MyEndPoint();

    // overrides from H323EndPoint
    H323Connection * CreateConnection(unsigned callReference);

    // new functions
    bool Initialise(PConfig & cfg, PConfigPage * rsrc);
    PString OnLoadEndPointStatus(const PString & htmlBlock);
    PString OnLoadCallStatus(const PString & htmlBlock);
    bool OnPostControl(const PStringToString & data, PHTML & msg);

  protected:
    MyGatekeeperServer gatekeeperServer;
};


class OpenGk : public PHTTPServiceProcess
{
  PCLASSINFO(OpenGk, PHTTPServiceProcess)

  public:
    OpenGk();
    void Main();
    bool OnStart();
    void OnStop();
    void OnControl();
    void OnConfigChanged();
    bool Initialise(const char * initMsg);

    static OpenGk & Current() { return (OpenGk &)PProcess::Current(); }

    MyEndPoint & GetEndPoint() const { return *endpoint; }

  protected:
#ifdef USE_OPAL
    OpalManager  manager;
#endif
    MyEndPoint * endpoint;
};


class MainStatusPage : public PServiceHTTPString
{
  PCLASSINFO(MainStatusPage, PServiceHTTPString);

  public:
    MainStatusPage(OpenGk & app, PHTTPAuthority & auth);
    
    virtual bool Post(
      PHTTPRequest & request,
      const PStringToString &,
      PHTML & msg
    );
  
  private:
    OpenGk & app;
};


#endif  // _OpenGk_MAIN_H


// End of File ///////////////////////////////////////////////////////////////
